@@ -1,6 +1,6 @@
 package URI::Fast;
 
-our $XS_VERSION = our $VERSION = '0.40_02';
+our $XS_VERSION = our $VERSION = '0.47_01';
 $VERSION =~ tr/_//;
 
 use utf8;
@@ -10,6 +10,7 @@ no strict 'refs';
 
 use Carp;
 use Exporter;
+use Scalar::Util qw(refaddr);
 
 require XSLoader;
 XSLoader::load('URI::Fast', $XS_VERSION);
@@ -122,6 +123,30 @@ sub add_param {
   $self->param($key, [$self->param($key), $val], $sep);
 }
 
+sub append {
+  my $self = shift;
+
+  foreach my $segment (@_) {
+    if ($segment =~ /^\?/) {
+      my $q = uri($segment)->query_hash;
+
+      foreach my $key (keys %$q) {
+        $self->add_param($key => $_)
+          foreach @{ $q->{$key} };
+      }
+    }
+    elsif (my ($frag) = $segment =~ /^#(.*)$/) {
+      $self->frag($frag);
+    }
+    else {
+      my @segments = ($self->path, split('/', $segment));
+      $self->path(\@segments);
+    }
+  }
+
+  return $self;
+}
+
 sub _cmp ($$) {
   if (defined $_[0]) {
     return if !defined $_[1];
@@ -173,7 +198,7 @@ sub compare {
 #-------------------------------------------------------------------------------
 sub relative {
   my ($self, $base) = @_;
-  my $rel = uri("$self");
+  my $rel = (ref $self)->new("$self");
 
   my $rpath = $self->path;
   ($_, $_, my $bpath) = uri_split("$base");
@@ -224,6 +249,72 @@ sub relative {
   return $rel;
 }
 
+sub _walk {
+  my ($ref, $sub) = @_;
+  my @stack = ($ref);
+  my %seen;
+
+  while (my $elt = shift @stack) {
+    my $ref = ref $elt;
+
+    if (defined $ref) {
+      my $addr = refaddr $elt;
+      next if exists $seen{ $addr };
+      $seen{ $addr } = 1;
+
+      if ($ref eq 'ARRAY') {
+        for (0 .. @$elt - 1) {
+          if (ref $elt->[$_]) {
+            push @stack, $elt->[$_];
+          }
+          elsif (defined $elt->[$_]) {
+            $elt->[$_] = $sub->( $elt->[$_] );
+          }
+        }
+      }
+      elsif ($ref eq 'HASH') {
+        foreach (keys %$elt) {
+          if (ref $elt->{$_}) {
+            push @stack, $elt->{$_};
+          }
+          elsif (defined $elt->{$_}) {
+            $elt->{$_} = $sub->( $elt->{$_} );
+          }
+        }
+      }
+      elsif ($ref eq 'SCALAR') {
+        push @stack, $$elt;
+      }
+    }
+  }
+
+  $ref;
+}
+
+sub escape_tree {
+  my ($ref, @args) = @_;
+  ref $ref || croak "escape_tree: reference expected";
+
+  my $sub = sub{
+    push @_, @args;
+    goto \&encode;
+  };
+
+  _walk($ref, $sub);
+}
+
+sub unescape_tree {
+  my ($ref, @args) = @_;
+  ref $ref || croak "unescape_tree: reference expected";
+
+  my $sub = sub{
+    push @_, @args;
+    goto \&decode;
+  };
+
+  _walk($ref, $sub);
+}
+
 =encoding UTF8
 
 =head1 NAME
@@ -283,6 +374,13 @@ Behaves (hopefully) identically to L<URI::Split>, but roughly twice as fast.
 
 See L</ENCODING>.
 
+=head1 CONSTRUCTOR
+
+If desired, both C<URI::Fast> and L<URI::Fast::IRI> may be instantiated using
+the default constructor, C<new>.
+
+  my $uri = URI::Fast->new('http://www.example.com');
+
 =head1 ATTRIBUTES
 
 All attributes serve as full accessors, allowing the URI segment to be both
@@ -345,11 +443,25 @@ L</auth>.
 In scalar context, returns the entire path string. In list context, returns a
 list of path segments, split by C</>.
 
+  my $uri = uri '/foo/bar';
+  my $path = $uri->path;  # "/foo/bar"
+  my @path = $uri->path;  # ("foo", "bar")
+
 The path may also be updated using either a string or an array ref of segments:
 
   $uri->path('/foo/bar');
   $uri->path(['foo', 'bar']);
 
+This differs from the behavior of L<URI/path_segments>, which considers the
+leading slash separating the path from the authority section to be an
+individual segment. If this behavior is desired, the lower level
+C<split_path_compat> is available. C<split_path_compat> (and its partner,
+C<split_path>), always return an array reference.
+
+  my $uri = uri '/foo/bar';
+  $uri->split_path;         # ['foo', 'bar'];
+  $uri->split_path_compat;  # ['', 'foo', 'bar'];
+
 =head2 query
 
 In scalar context, returns the complete query string, excluding the leading
@@ -466,6 +578,20 @@ An optional second parameter may be specified to control the separator
 character used when updating the query string. The same caveats apply with
 regard to normalization of the query string separator.
 
+=head2 append
+
+Serially appends path segments, query strings, and fragments, to the end of the
+URI. Each argument is added in order. If the segment begins with C<?>, it is
+assumed to be a query string and it is appended using L</add_param>. If the
+segment begins with C<#>, it is treated as a fragment, replacing any existing
+fragment. Otherwise, the segment is treated as a path fragment and appended to
+the path.
+
+  my $uri = uri 'http://www.example.com/foo?k=v';
+  $uri->append('bar', 'baz/bat', '?k=v1&k=v2', '#fnord', 'slack');
+  # 'http://www.example.com/foo/bar/baz/bat/slack?k=v&k=v1&k=v2#fnord'
+
+
 =head2 to_string
 
 =head2 as_string
@@ -590,28 +716,48 @@ due to some pedantic whining on the part of
 L<BGRIMM|https://metacpan.org/author/BGRIMM>, they have been renamed to
 C<uri_encode> and C<uri_decode>.
 
+=head2 escape_tree
+
+=head2 unescape_tree
+
+Traverses a data structure, escaping or unescaping I<defined> scalar values in
+place. Accepts a reference to be traversed. Any further parameters are passed
+unchanged to L</encode> or L</decode>. Croaks if the input to escape/unescape
+is a non-reference value.
+
+  my $obj = {
+    foo => ['bar baz', 'bat%fnord'],
+    bar => {baz => 'bat%bat'},
+    baz => undef,
+    bat => '',
+  };
+
+  URI::Fast::escape_tree($obj);
+
+  # $obj is now:
+  {
+    foo => ['bar%20baz', 'bat%25fnord'],
+    bar => {baz => 'bat%25bat'},
+    baz => undef,
+    bat => '',
+  }
+
+  URI::Fast::unescape_tree($obj); # $obj returned to original form
+
+  URI::Fast::escape_tree($obj, '%'); # escape but allow "%"
+
+  # $obj is now:
+  {
+    foo => ['bar%20baz', 'bat%fnord'],
+    bar => {baz => 'bat%bat'},
+    baz => undef,
+    bat => '',
+  }
+
 =head1 SPEED
 
 See L<URI::Fast::Benchmarks>.
 
-=head1 FUTURE PLANS
-
-Some thoughts on future development for this module.
-
-=over
-
-=item Support for arbitrary binary data in query string
-
-Currently, queries are restricted to keys or key/value pairs and decoded into
-utf8 strings.
-
-=item Compat module
-
-Add a compatibility layer with URI that implements what functionality is
-possible and croaks loudly otherwise.
-
-=back
-
 =head1 SEE ALSO
 
 =over
@@ -658,6 +804,10 @@ fun of me for naming certain methods too generically.
 
 =item Tim Vroom (VROOM)
 
+=item Des Daignault (NAWGLAN)
+
+=item Josh Rosenbaum
+
 =back
 
 =head1 AUTHOR
