@@ -3,7 +3,8 @@ package URI::Fast::IRI;
 use strict;
 use warnings;
 
-our $VERSION = '0.37';
+require URI::Fast;
+our $VERSION = '0.47_01';
 
 our @ISA = qw(URI::Fast);
 
@@ -27,4 +28,6 @@ programming language system itself.
 
 =cut
 
+sub clone { URI::Fast::IRI->new($_[0]) }
+
 1;
